module Main where

import System.Environment
import Data.List

main = do
  [sizestr, ustr] <- getArgs
  let [size, u] = map read [sizestr, ustr]
  print (choose (2 * size) size)
--  print (ascchoices (2*size) size)
  print (nlimitchoices (2*size) size u)
--  print (nlimitdeadends (2*size) size u)
--  print (nlimitcalls (2*size) size u)
--  print (limitchoices (2*size) size u)


-- All the numerical quantities counting choices are of the Integer type.  The
-- compiler (GHC) seems to get this right automatically


-- Binomial coefficient: number of choices of m out of n
choose _ 0 = 1
choose 0 _ = 0
choose n m = (choose (n-1) (m-1)) * n `div` m


-- All choices m out of n, sorted in ascending order, 0-based.
-- We loop not over the actual chosen indices (ranks) but over the number of
-- indices left out between successive chosen ones, which is d.  To get the
-- numbers right in the result, we add (d+1) using two maps.
ascchoices n 0 = [[]]
ascchoices n m = foldl' (++) [] [ map (d:) (map (map (+(d+1))) (ascchoices (n-d-1) (m-1))) | d <- [0..(n-m)] ]


-- All choices m out of n up to the Mann-Whitney u given.
-- d is the number of elements of the other sample between successive values of
-- the one we are interested in, so the share of u we have to spare is reduced
-- by the number of remaining elements times d.
limitchoices n 0 u =
  if u < 0
    then []
    else [[]]
limitchoices n m u =
  if u < 0
    then []
    else foldl' (++) [] [ map (d:) (map (map (+(d+1))) (limitchoices (n-d-1) (m-1) (u-m*d))) | d <- [0..(n-m)] ]


-- The number of choices m out of n with at most the given Mann-Whitney U.
-- The important thing is to abort once the sum of ranks is sure to get too
-- large (first case in both definitions).
nlimitchoices :: Int -> Int -> Int -> Integer
nlimitchoices n 0 u | u < 0 = 0 | otherwise = 1
nlimitchoices n m u
  | u < 0 = 0
  | otherwise = foldl' (+) 0 [ nlimitchoices (n-d-1) (m-1) (u-m*d) | d <- [0..(n-m)] ]


-- The number of dead ends when searching for u-limited choices, ie the number
-- of times our u budget runs out without finding a valid coice of samples.
nlimitdeadends n 0 u | u < 0 = 1 | otherwise = 0
nlimitdeadends n m u
  | u < 0 = 1
  | otherwise = foldl' (+) 0 [ nlimitdeadends (n-d-1) (m-1) (u-m*d) | d <- [0..(n-m)] ]


-- The number of recursive calls required to obtain the results.
nlimitcalls n 0 u = 1
nlimitcalls n m u
  | u < 0 = 1
  | otherwise = foldl' (+) 1 [ nlimitcalls (n-d-1) (m-1) (u-m*d) | d <- [0..(n-m)] ]


allways :: (a -> b -> c) -> [a] -> [b] -> [c]
allways _ [] _ = []
allways _ _ [] = []
allways f (x:xs) ys = (map (f x) ys) ++ (allways f xs ys)

