#!/usr/bin/perl -w

use strict;
use bigint;

$#ARGV == 1 or die "usage: perl_mannwhitney_i.pl <size of both samples> <U>";

my ($size, $Ugiven)= @ARGV;

sub choose
{
  my ($n, $k)= @_;

  return 1 if $k == 0;
  return 0 if $n == 0;
  return choose($n-1, $k-1) * $n / $k;
}

# compute integral of normal distribution with variance 1 from x to infinity
# according to Abramowitz and Stegun
sub sig_as
{
  my ($x)= @_;
  my @coeffs= ( 0.319381530, -0.356563782, 1.781477937, -1.821255978, 1.330274429 );
  my $PI= 3.1415926535897932384626433832795029;

  my $poly= 0;
  my $t= 1/(1+0.2316419*$x);
  my $tpow= $t;
  for my $coef (@coeffs) {
    $poly += $coef * $tpow;
    $tpow *= $t;
  }
  return $poly * exp(-$x*$x/2) / sqrt(2*$PI);
}

sub nlimitchoices_i
{
  my ($n, $m, $u)= @_;
  my $sum= 0;
  my @dstack;

  return 0 if $u < 0;
  return 1 if $m == 0;
  push @dstack, 0;
  while( @dstack ) {
    while( $m > 1 ) {	# descend
      push @dstack, 0;
      --$n;
      --$m;
    }
    ++$sum;
    while( @dstack ) {	# backtrack
      ++$dstack[-1];
      --$n;
      $u -= $m;
      last if $n >= $m && $u >= 0;
      $n += $dstack[-1]+1;
      $u += $m*$dstack[-1];
      ++$m;
      pop @dstack;
    }
  }
  return $sum;
}

my $nchoices= choose(2*$size, $size);
print "$nchoices\n";

my $Usmaller= nlimitchoices_i(2*$size, $size, $Ugiven);
print "$Usmaller\n";

# print "probability of a smaller U (= one-tailed significance): ",
# 	$Usmaller / $nchoices, "\n";

# my $z= abs($Ugiven - $size*$size/2) / sqrt($size*$size*(2*$size+1)/12);
# print "significance according to Gaussian approximation: ", sig_as($z), "\n";

